/*
 *
 *    OPEN-XCHANGE legal information
 *
 *    All intellectual property rights in the Software are protected by
 *    international copyright laws.
 *
 *
 *    In some countries OX, OX Open-Xchange, open xchange and OXtender
 *    as well as the corresponding Logos OX Open-Xchange and OX are registered
 *    trademarks of the OX Software GmbH group of companies.
 *    The use of the Logos is not covered by the GNU General Public License.
 *    Instead, you are allowed to use these Logos according to the terms and
 *    conditions of the Creative Commons License, Version 2.5, Attribution,
 *    Non-commercial, ShareAlike, and the interpretation of the term
 *    Non-commercial applicable to the aforementioned license is published
 *    on the web site http://www.open-xchange.com/EN/legal/index.html.
 *
 *    Please make sure that third-party modules and libraries are used
 *    according to their respective licenses.
 *
 *    Any modifications to this package must retain all copyright notices
 *    of the original copyright holder(s) for the original code used.
 *
 *    After any such modifications, the original and derivative code shall remain
 *    under the copyright of the copyright holder(s) and/or original author(s)per
 *    the Attribution and Assignment Agreement that can be located at
 *    http://www.open-xchange.com/EN/developer/. The contributing author shall be
 *    given Attribution for the derivative code and a license granting use.
 *
 *     Copyright (C) 2016-2020 OX Software GmbH
 *     Mail: info@open-xchange.com
 *
 *
 *     This program is free software; you can redistribute it and/or modify it
 *     under the terms of the GNU General Public License, Version 2 as published
 *     by the Free Software Foundation.
 *
 *     This program is distributed in the hope that it will be useful, but
 *     WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY
 *     or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 *     for more details.
 *
 *     You should have received a copy of the GNU General Public License along
 *     with this program; if not, write to the Free Software Foundation, Inc., 59
 *     Temple Place, Suite 330, Boston, MA 02111-1307 USA
 *
 */

package com.openexchange.guard.file.storage;

import java.util.Collection;
import com.openexchange.file.storage.Document;
import com.openexchange.file.storage.File;
import com.openexchange.file.storage.File.Field;

/**
 * {@link EncryptedFileRecognizer} defines how to recognize encrypted files.
 *
 * @author <a href="mailto:benjamin.gruedelbach@open-xchange.com">Benjamin Gruedelbach</a>
 * @since v7.8.2
 */
public interface EncryptedFileRecognizer {

    /**
     * Checks if the given file is an encrypted document
     *
     * @param file The file
     * @return True if the file is an encrypted file, false otherwise
     */
    boolean isEncrypted(File file);

    /**
     * Marks a specific file as encrypted
     *
     * @param file The file to mark as encrypted
     * @return A collection of {@link com.openexchange.file.storage.File.Field} which are relevant for marking a file as encrypted.
     */
    Collection<Field> markEncrypted(File file);

    /**
     * Marks a specific document as encrypted
     *
     * @param document The document to mark as encrypted
     * @return A collection of {@link com.openexchange.file.storage.File.Field} which are relevant for marking a file as encrypted.
     */
    Collection<Field> markEncrypted(Document document);

    /**
     * Marks a specific file as decrypted
     *
     * @param file The file to mark as decrypted
     * @return A collection of {@link com.openexchange.file.storage.File.Field} which are relevant for marking a file as decrypted.
     */
    Collection<Field> markDecrypted(File file);

    /**
     * Marks a specific document as decrypted
     *
     * @param document The document to mark as decrypted
     * @return A collection of {@link com.openexchange.file.storage.File.Field} which are relevant for marking a document as decrypted.
     */
    Collection<Field> markDecrypted(Document document);
}
